#!/usr/bin/python
# -*- coding: utf-8 -*-

import gi
gi.require_version('Gtk', '3.0')
gi.require_version('AppIndicator3', '0.1')
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import AppIndicator3
import os
import sys
import signal
import re
from cgaldapsearch import LDAP, PasswordIncorrect
import gettext
import locale

current_locale, encoding = locale.getdefaultlocale()
locale_path = '/usr/share/locale'

language = gettext.translation ('cga-password-reset', locale_path, [current_locale] )
language.install()
_ = language.ugettext

try:
    p = LDAP(os.environ['USER'])
    if p.isLDAPed():
        LOGIN = os.environ['USER']

except:
    print "User not allowed"
    sys.exit()

title = _(u"Cambiar contraseña de %s") % LOGIN

class PasswordChanger(Gtk.Window):

    VALIDPASSWORD_REGEX=[r"^[A-Za-z0-9_-]+$"]

    def __init__(self):
        """ Class init event """
        # Setup the window with title and border width.
        Gtk.Window.__init__(self, type=Gtk.WindowType.TOPLEVEL,
                                  title=title,
                                  resizable=False,
                                  border_width=10)
        self.override_background_color(Gtk.StateType.NORMAL, Gdk.RGBA(.9,.9,.9,1))

        # Boolean for correct password.
        self.valid_password = False

        # Create a Vertical child box.
        vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 4)
        buttonbox = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 6)
        newpassbox = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
        repeatpwdbox = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)


        # Labels
        title_label = Gtk.Label(_(u"Rellene los campos solicitados para cambiar de contraseña"))
        samepass_label = Gtk.Label("")

        # Create the entries
        username_entry = Gtk.Entry()
        username_entry.set_placeholder_text(LOGIN)
        username_entry.set_property("editable", False)
        username_entry.set_sensitive(False)
        oldpassword_entry = Gtk.Entry()
        oldpassword_entry.set_placeholder_text(_(u"Contraseña antigua"))
        #oldpassword_entry.override_background_color(Gtk.StateType.NORMAL, Gdk.RGBA(0.0, 1.0, 0.0, 1.0))

        oldpassword_entry.set_visibility(False)
        newpassword_entry = Gtk.Entry()
        newpassword_entry.set_placeholder_text(_(u"Nueva contraseña"))
        newpassword_entry.set_visibility(False)
        repeatpwd_entry = Gtk.Entry()
        repeatpwd_entry.set_placeholder_text(_(u"Repetir contraseña"))
        repeatpwd_entry.set_visibility(False)

        # The strong password bar
        strongpass_progressbar = Gtk.ProgressBar()
        strongpassbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 6)
        strongpassbox.modify_bg(Gtk.StateFlags.PRELIGHT, Gdk.color_parse("green"))

        # Two Buttons
        ok_button = Gtk.Button(_(u"Aceptar"))
        ok_button.set_sensitive(False)
        cancel_button = Gtk.Button(_(u"Cancelar"))

        # Connectors
        ok_button.connect("clicked", self.on_okbutton_clicked)
        cancel_button.connect("clicked", Gtk.main_quit)
        repeatpwd_entry.connect("changed", self.on_repeatpwd_changed)
        newpassword_entry.connect("changed", self.on_newpassword_changed)

        # Self associations
        self.ok_button = ok_button
        self.cancel_button = cancel_button
        self.oldpassword_entry = oldpassword_entry
        self.newpassword_entry = newpassword_entry
        self.repeatpwd_entry = repeatpwd_entry
        self.samepass_label = samepass_label
        self.strongpass_progressbar = strongpass_progressbar

        # Packing the form
        vbox.pack_start(title_label, False, False, 10)
        vbox.pack_start(username_entry, False, False, 0)
        vbox.pack_start(oldpassword_entry, False, False, 0)
        newpassbox.pack_start(newpassword_entry, False, False, 0)
        strongpassbox.pack_start(strongpass_progressbar, True, True, 10)
        newpassbox.pack_start(strongpassbox, True, True, 10)
        vbox.pack_start(newpassbox, False, False, 0)
        repeatpwdbox.pack_start(repeatpwd_entry, False, False, 0)
        repeatpwdbox.pack_start(samepass_label, False, False, 10)
        vbox.pack_start(repeatpwdbox, False, False, 0)
        buttonbox.pack_end(ok_button, False, False, 0)
        buttonbox.pack_end(cancel_button, False, False, 0)
        vbox.pack_start(buttonbox, False, False, 10)

        # Add the child box to the window
        self.add(vbox)
        self.connect("destroy", Gtk.main_quit)
        signal.signal(signal.SIGINT, signal.SIG_DFL)

        # Some style
        css = """
GtkEntry {
    border-radius: 5px;
    border-style: solid;
    border-width: 1px;
}
GtkProgressBar {
    border-width: 1px;
}
GtkProgressBar#0 {
    background-color: #FF0000;
}
        """
        style_provider = Gtk.CssProvider()
        style_provider.load_from_data(css)

        Gtk.StyleContext.add_provider_for_screen(
            Gdk.Screen.get_default(),
            style_provider,
            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
        )

    def on_newpassword_changed(self, widget):
        strength = 0
        special_chars = "!\"#$%&'()*+,-./:;<=>?@[\]^_`{|}~"

        rules = [lambda s: any(x.isupper() for x in s),        # at least one uppercase
                 lambda s: any(x.islower() for x in s),        # at least one lowercase
                 lambda s: any(x.isdigit() for x in s),        # at least one digit
                 lambda s: any(x in special_chars for x in s), # at least one symbol
                 lambda s: len(s) >= 6,                        # at least 6 characters
                 lambda s: len(s) >= 10                        # at least 10 characters
        ]

        for rule in rules:
            if rule(self.newpassword_entry.get_text()):
                strength += 1

        if strength == 0:
            self.strongpass_progressbar.hide()
        else:
            self.strongpass_progressbar.show()
            fraction = 0.166666667 * float(strength)
            self.strongpass_progressbar.set_fraction(float(fraction))

        for r in self.VALIDPASSWORD_REGEX:
            # print regexp
            if re.match(r, self.newpassword_entry.get_text()):
                self.strongpass_progressbar.show()
                if self.newpassword_entry.get_text() != self.oldpassword_entry.get_text():
                    self.samepass_label.set_text(_(u"Las contraseñas no coinciden"))
                else:
                    self.samepass_label.set_text("")
                self.valid_password = True
            else:
                self.strongpass_progressbar.hide()
                self.samepass_label.set_text(_(u"Contraseña incorrecta"))
                self.valid_password = False

            if self.newpassword_entry.get_text() != self.repeatpwd_entry.get_text():
                self.ok_button.set_sensitive(False)
            elif self.oldpassword_entry:
                self.ok_button.set_sensitive(True)


    def on_repeatpwd_changed(self, widget):
        if self.newpassword_entry.get_text() != self.repeatpwd_entry.get_text():
            self.samepass_label.set_text(_(u"Las contraseñas no coinciden"))
            self.ok_button.set_sensitive(False)

        elif self.valid_password:
            self.samepass_label.set_text("")
            if self.oldpassword_entry:
                self.ok_button.set_sensitive(True)

    def on_okbutton_clicked(self, widget):
        oldpassword = self.oldpassword_entry.get_text()
        password = self.newpassword_entry.get_text()

        p = LDAP(LOGIN)
        if p.isLDAPed():
            try:
                p.passwd(oldpassword, password)
                dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.INFO,
                                           Gtk.ButtonsType.OK,
                                           _(u"Contraseña cambiada correctamente"))
                dialog.run()
                dialog.destroy()
                Gtk.main_quit()
                # return True

            except PasswordIncorrect:
                dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR,
                                           Gtk.ButtonsType.OK,
                                           _(u"La contraseña antigua no coincide"))
                dialog.run()
                dialog.destroy()
                return False

            except:
                dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR,
                                           Gtk.ButtonsType.OK,
                                           _(u"No se ha podido cambiar la contraseña"))
                dialog.run()
                dialog.destroy()
                return False



        else:
            return False

def show_gui():
    pid = str(os.getpid())
    pidfile = "/tmp/cgapasswordresetgui%s.pid" % os.environ['USER']

    if os.path.isfile(pidfile):
        print "%s already exists, exiting" % pidfile
        sys.exit()

    file(pidfile, 'w').write(pid)

    try:
        window = PasswordChanger()
        window.set_position(Gtk.WindowPosition.CENTER)
        window.show_all()
        window.strongpass_progressbar.hide()
        window.cancel_button.grab_focus()

        Gtk.main()

    finally:
        os.unlink(pidfile)

if __name__ == '__main__':
    show_gui()
